from django.contrib import admin
from photos.models import Photo, Album
from django.utils.translation import ugettext as _
from django.template.loader import get_template
from django.template import Context

class BaseAdmin():
    def queryset(self, request):
        qs = super(PhotoAdmin, self).queryset(request)
        if request.user.is_superuser:
            return qs
        return qs.filter(user=request.user)

    def save_model(self, request, obj, form, change):
        obj.user = request.user
        obj.save()

class PhotoAdmin(admin.ModelAdmin, BaseAdmin):
    list_filter = ('display', 'timestamp_created', )
    list_display = ('name', 'display', 'timestamp_created', 'get_thumbnail', )
    date_hierarchy = 'timestamp_created'
    description = _('Manage your uploaded photos.')
    
    def get_thumbnail(self, obj):
        template = get_template('photos/admin_thumbnail.html')
        context = Context({'photo': obj})
        return template.render(context)
    
    get_thumbnail.allow_tags = True
    get_thumbnail.short_description = _('Thumbnail')

admin.site.register(Photo, PhotoAdmin)

class AlbumAdmin(admin.ModelAdmin, BaseAdmin):
    list_filter = ('display', 'timestamp_created', )
    date_hierarchy = 'timestamp_created'
    description = _('Manage your galleries.')
    filter_horizontal = ('photos', )
    
admin.site.register(Album, AlbumAdmin)




from photos.models import Photo, Album
from django.contrib.auth.models import User
from django.views.generic.list_detail import object_list, object_detail
from django.shortcuts import get_object_or_404

def album_list(request, user_id=0, username=None, *args, **kwargs):
    qs = Album.objects.filter(display=True)
    user = None
    
    if user_id > 0:
        user = get_object_or_404(User, id=user_id)
        qs = qs.filter(user=user)
    elif username is not None:
        user = get_object_or_404(User, username=username)
        qs = qs.filter(user=user)
        
    if user:
        extra_context = kwargs.get('extra_context', {})
        extra_context.update({'user': user})
        kwargs.update({'extra_context': extra_context})
        
    return object_list(request, queryset=qs, *args, **kwargs)

def photo_list(request, user_id=0, username=None, *args, **kwargs):
    qs = Photo.objects.filter(display=True)
    user = None
    
    if user_id > 0:
        user = get_object_or_404(User, id=user_id)
        qs = qs.filter(user=user)
    elif username is not None:
        user = get_object_or_404(User, username=username)
        qs = qs.filter(user=user)
        
    if user:
        extra_context = kwargs.get('extra_context', {})
        extra_context.update({'user': user})
        kwargs.update({'extra_context': extra_context})
    
    return object_list(request, queryset=qs, *args, **kwargs)

def photo_detail(request, object_id, album_id, *args, **kwargs):
    album = get_object_or_404(Album, id=album_id)
    
    extra_context = kwargs.get('extra_context', {})
    extra_context.update({'album': album})
    kwargs.update({'extra_context': extra_context})
        
    return object_detail(request, object_id=object_id, *args, **kwargs)


from photos.views import album_list, photo_list, photo_detail
from django.views.generic.list_detail import object_detail
from django.conf.urls.defaults import *
from django.conf import settings
from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    url(r'^photos/', include('photos.urls')),
    (r'^static/(?P<path>.*)$', 'django.views.static.serve', {'document_root': settings.MEDIA_ROOT}),
    (r'^admin/', include(admin.site.urls))
)





from django.db import models
from taggit.managers import TaggableManager
from django.utils.translation import ugettext as _
from django.contrib.auth.models import User
from os.path import join, abspath, dirname, basename
import datetime
from hashlib import sha224
import random
from django.db.models.signals import pre_delete, pre_save
from shutil import rmtree

def get_upload_to(instance, filename):
    dirs = datetime.datetime.now().strftime("photos/%y/%j/%H/%M/%S")
    praefix = instance.hash
    return join(dirs, praefix, filename)

def get_hash():
    return sha224(str(random.random())).hexdigest()

class Photo(models.Model):
    photo = models.ImageField(_('Photo'), upload_to=get_upload_to, height_field='height', width_field='width', max_length=250)
    height = models.PositiveIntegerField(_('Height'), editable=False)
    width = models.PositiveIntegerField(_('Width'), editable=False)
    display = models.BooleanField(_('Display'), default=True, help_text=_('Enable to display this photo.'))
    name = models.CharField(_('Name'), max_length=250, help_text=_('If no name is given, it is created from filename.'), blank=True)
    description = models.TextField(_('Description'), blank=True, null=True)
    user = models.ForeignKey(User, editable=False, blank=True, null=True)
    tags = TaggableManager()
    timestamp_created = models.DateTimeField(_('Timestamp created'), auto_now_add=True, editable=False)
    timestamp_edited = models.DateTimeField(_('Timestamp edited'), auto_now=True, editable=False)
    hash = models.CharField(_('Hash'), editable=False, default=get_hash, max_length=64)
    
    def get_albums(self):
        return self.album_set.filter(display=True)
    
    def __unicode__(self):
        return self.name
    
    class Meta(object):
        verbose_name = _('Photo')
        verbose_name_plural = _('Photos')
        ordering = ('-timestamp_created', )
        
    @models.permalink
    def get_absolute_url(self):
        return ('photo-detail', [self.id])
        
class Album(models.Model):
    display = models.BooleanField(_('Display'), default=True, help_text=_('Disable to not display this gallery.'))
    name = models.CharField(_('Name'), max_length=250)   
    description = models.TextField(_('Description'), blank=True, null=True)
    photos = models.ManyToManyField(Photo, verbose_name=_('Photos'))
    tags = TaggableManager()
    user = models.ForeignKey(User, editable=False, blank=True, null=True)
    timestamp_created = models.DateTimeField(_('Timestamp created'), auto_now_add=True, editable=False)
    timestamp_edited = models.DateTimeField(_('Timestamp edited'), auto_now=True, editable=False)
    
    def get_photos(self):
        return self.photos.filter(display=True)
    
    def __unicode__(self):
        return self.name
    
    class Meta(object):
        verbose_name = _('Album')
        verbose_name_plural = _('Albums')
        ordering = ('-timestamp_created', )
        
    @models.permalink
    def get_absolute_url(self):
        return ('album-detail', [self.id])

def pre_save_photo(*args, **kwargs):
    instance = kwargs['instance']
    path = instance.photo.path
    if not instance.name:
        name = basename(path).rsplit(".", 1)[0]
        instance.name = name
    
pre_save.connect(pre_save_photo, sender=Photo)

def pre_delete_photo(*args, **kwargs):
    instance = kwargs['instance']
    try:
        path = instance.photo.path
    except ValueError, e:
        pass # silently
    else:
        rmtree(path=abspath(dirname(path)), ignore_errors=True)

pre_delete.connect(pre_delete_photo, sender=Photo)
    


from photos.views import album_list, photo_list, photo_detail
from django.views.generic.list_detail import object_detail
from django.conf.urls.defaults import *
from photos.models import Photo, Album

ALBUM_DETAIL_DICT = {'queryset': Album.objects.filter(display=True)}
PHOTO_DETAIL_DICT = {'queryset': Photo.objects.filter(display=True)}

urlpatterns = patterns('',
    url(r'^albums/$', album_list, name='albums'),
    url(r'^albums/(?P<user_id>\d+)/$', album_list, name='albums-userid'),
    url(r'^albums/(?P<username>.+)/$', album_list, name='albums-username'),
    url(r'^album/(?P<object_id>\d+)/$', object_detail, ALBUM_DETAIL_DICT, name='album-detail'),
    url(r'^photos/$', photo_list, name='photos'),
    url(r'^photos/(?P<user_id>\d+)/$', photo_list, name='photos-userid'),
    url(r'^photo/(?P<object_id>\d+)/$', object_detail, PHOTO_DETAIL_DICT, name='photo-detail'),
    url(r'^photos/(?P<album_id>\d+)/(?P<object_id>\d+)/$', photo_detail, PHOTO_DETAIL_DICT, name='photo-album-detail'),
    url(r'^photos/(?P<username>.+)/', photo_list, name='photos-username')
)



from setuptools import setup, find_packages
import sys, os

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
NEWS = open(os.path.join(here, 'NEWS.txt')).read()


version = '0.2.8'

install_requires = [
    'Django',
    'django-taggit',
    'django-templatetag-sugar',
    'easy_thumbnails',   
    'pil'
]


setup(name='django-photos',
    version=version,
    description="This is a simple photo-app for django.",
    long_description=README + '\n\n' + NEWS,
    classifiers=[
                 "Development Status :: 3 - Alpha",
                 "Environment :: Web Environment",
                 "Framework :: Django",
                 "Intended Audience :: Developers",
                 "License :: OSI Approved :: BSD License",
                 "Natural Language :: English",
                 "Programming Language :: Python",
                 "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
                 
    ],
    keywords='django photo gallery',
    author='Julian Moritz',
    author_email='jumo@gmx.de',
    url='http://www.bitbucket.org/feuervogel/django-photos',
    license='BSD',
    packages=find_packages('src'),
    package_dir = {'': 'src'},include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    entry_points={
        'console_scripts':
            ['django-photos=djangophotos:main']
    },
    test_suite='runtests.runtests'
)



from django import template

from templatetag_sugar.register import tag
from templatetag_sugar.parser import Name, Variable, Constant, Optional, Model

register = template.Library()

@tag(register, [Variable(), Variable(), Constant('as'), Name()])
def get_previous(context, album, photo, asvar):
    prev = None
    for tmp in album.get_photos():
        if tmp == photo:
            context[asvar] = prev
            return ''
        else:
            prev = tmp
    
    return ''

@tag(register, [Variable(), Variable(), Constant('as'), Name()])
def get_next(context, album, photo, asvar):
    flag = False
    for tmp in album.get_photos():
        if flag == True:
            context[asvar] = tmp
            return ''
        if tmp == photo:
            flag = True
                
    return ''

@tag(register, [Constant('as'), Name()])
def get_photos(context, asvar):
    context[asvar] = Photos.objects.filter(display=True)
    return ''

@tag(register, [Constant('as'), Name()])
def get_albums(context, asvar):
    context[asvar] = Albums.objects.filter(display=True)
    return ''


from django.test import TestCase
from django.test.client import Client
from django.core.urlresolvers import reverse
from django.contrib.auth.models import User
from photos.models import Photo, Album

class TemplateTagTest(TestCase):
    pass

class ViewTest(TestCase):
    def setUp(self):
        user, created = User.objects.get_or_create(username='anna')
        album = Album()
        album.user = user
        album.save()
        photo1 = Photo(photo='a.jpg')
        photo1.user = user
        photo1.save()
        
        photo2 = Photo(photo='b.jpg')
        photo2.user = user
        photo2.save()
        
        album.photos.add(photo1)
        album.photos.add(photo2)
        album.save()        
    
    def check_url(self, url, status_code=200):
        client = Client()
        response = client.get(url)
        msg = '%s != %s in %s' % (response.status_code, status_code, url)
        self.assertEqual(response.status_code, status_code, msg)
    
    def test_album_list(self):
        url = reverse('albums')
        self.check_url(url, 200)
        
    def test_album_list_userid(self):
        user = User.objects.latest('id')
        url = reverse('albums-userid', args=(user.id, ))
        self.check_url(url, 200)
        
    def test_album_list_username(self):
        user = User.objects.latest('id')
        url = reverse('albums-username', args=(), kwargs={'username': user.username})
        self.check_url(url, 200)
        
    def test_album_detail(self):
        album = Album.objects.latest('id')
        url = reverse('album-detail', args=(album.id, ))
        self.check_url(url, 200)
        
    def test_photo_list(self):
        url = reverse('photos')
        self.check_url(url, 200)
        
    def test_photo_list_userid(self):
        user = User.objects.latest('id')
        url = reverse('photos-userid', args=(user.id, ))
        self.check_url(url, 200)
        
    def test_photo_list_username(self):
        user = User.objects.latest('id')
        url = reverse('photos-username', args=(), kwargs={'username': user.username})
        self.check_url(url, 200)
        
    def test_photo_detail(self):
        photo = Photo.objects.latest('id')
        url = reverse('photo-detail', args=(photo.id, ))
        self.check_url(url, 200)
        
    def test_photo_name(self):
        photo = Photo.objects.get(id=1)
        self.assertEqual(photo.name, 'a')
    